--[[ Handle the Raid Roster ]]

local Triage = _G.Triage
Triage.Roster = Triage:NewModule("Roster", "AceEvent-3.0")
local Roster = Triage.Roster

local raid_ids = {};
local raid_members = {};

Roster.debugFrame = ChatFrame1
Roster.debugging = true


function Roster:Debug(str, ...)
	if not self.debugging then return end

	if not str or str:len() == 0 then
		str = "nil"
	elseif str:find("%%") and select("#", ...) > 0 then
		str = str:format(...)
	elseif select("#", ...) > 0 then
		str = string.join(" ", str, tostringall(...))
	end

	self.debugFrame:AddMessage("|c933f9fffTriage Roster:|r " .. str)
end



--[[ Primary module callbacks, initialize, enable, disable ]]
function Roster:OnInitialize()
	-- Called at module load
	self:Debug("OnInitialize")
	for i = 1, MAX_RAID_MEMBERS do
		table.insert( raid_ids, ("raid%d"):format(i) )
	end
	self:OnEnable()
end



function Roster:OnEnable()
	-- Called when the module is enabled
	self:Debug("OnEnable")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("UNIT_NAME_UPDATE")

	self:Update()
end



function Roster:OnDisable()
	self:Debug("OnDisable")
	-- Called when the module is disabled
end



--[[ Event handlers ]]
function Roster:PLAYER_ENTERING_WORLD()
	self:Debug("PLAYER_ENTERING_WORLD")
	self:Update()
end



function Roster:PARTY_MEMBERS_CHANGED()
	self:Debug("PARTY_MEMBERS_CHANGED")
	self:Update()
end



function Roster:RAID_ROSTER_UPDATE()
	self:Debug("RAID_ROSTER_UPDATE")
	self:Update()
end



function Roster:UNIT_NAME_UPDATE()
	self:Debug("UNIT_NAME_UPDATE")
	self:Update()
end



--[[ Update the raid member list ]]
function Roster:Update()
	-- self:Debug("Update")

	if GetNumRaidMembers() == 0 then return end

	local updated = false
	local update_members = {}
	
	for index, id in ipairs(raid_ids) do
		-- self:Debug("updating "..id)
		if id and UnitExists(id) then
			local name, realm = UnitName(id)
			table.insert( update_members, name )
		end
	end

	table.sort( update_members )
	if #raid_members ~= #update_members then
		raid_members = update_members
		updated = true
		self:Debug("members size changed")
	else
		for i = 1, #raid_members do
			if raid_members[i] ~= update_members[i] then
				self:Debug(raid_members[i].." ~= "..update_members[i])
				raid_members = update_members
				updated = true
				break
			end
		end
	end

	-- Debugging
	if updated then
	   raid_list = ""
	   for i = 1, #raid_members do
	       raid_list = raid_list.." "..raid_members[i]
	   end
	   self:Debug(raid_list)
	end

	if updated then
		self:Debug("Sending RosterUpdated message")
		self.SendMessage("Triage_RosterUpdated")
	end
end



function Roster:GetRoster()
	return raid_members
end